"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SortExecutor = exports.FilterExecutor = exports.TransformExecutor = void 0;
const algorithm_1 = require("@lumino/algorithm");
const moment = __importStar(require("moment"));
/**
 * An object that defines a data transformation executor.
 */
class TransformExecutor {
}
exports.TransformExecutor = TransformExecutor;
/**
 * A transformation that filters a single field by the provided operator and
 * value.
 *
 * Note: This is a WIP
 */
class FilterExecutor extends TransformExecutor {
    /**
     * Create a new transformation.
     *
     * @param options - The options for initializing the transformation.
     */
    constructor(options) {
        super();
        this._options = options;
    }
    /**
     * Apply a transformation to the provided data.
     *
     * @param input - The data to be operated on.
     */
    apply(input) {
        let filterFunc;
        switch (this._options.operator) {
            case '>':
                filterFunc = (item) => {
                    if (['date', 'datetime', 'time'].includes(this._options.dType)) {
                        const target = moment.default.utc(item[this._options.field]);
                        const value = moment.default.utc(this._options.value);
                        return target.isAfter(value, 'day');
                    }
                    return item[this._options.field] > this._options.value;
                };
                break;
            case '<':
                filterFunc = (item) => {
                    if (['date', 'datetime', 'time'].includes(this._options.dType)) {
                        const target = moment.default.utc(item[this._options.field]);
                        const value = moment.default.utc(this._options.value);
                        return target.isBefore(value, 'day');
                    }
                    return item[this._options.field] < this._options.value;
                };
                break;
            case '<=':
                filterFunc = (item) => {
                    if (['date', 'datetime', 'time'].includes(this._options.dType)) {
                        const target = moment.default.utc(item[this._options.field]);
                        const value = moment.default.utc(this._options.value);
                        return target.isSameOrBefore(value, 'day');
                    }
                    return item[this._options.field] <= this._options.value;
                };
                break;
            case '>=':
                filterFunc = (item) => {
                    if (['date', 'datetime', 'time'].includes(this._options.dType)) {
                        const target = moment.default.utc(item[this._options.field]);
                        const value = moment.default.utc(this._options.value);
                        return target.isSameOrAfter(value, 'day');
                    }
                    return item[this._options.field] >= this._options.value;
                };
                break;
            case '=':
                filterFunc = (item) => {
                    if (['date', 'datetime', 'time'].includes(this._options.dType)) {
                        const target = moment.default.utc(item[this._options.field]);
                        const value = moment.default.utc(this._options.value);
                        return target.isSame(value);
                    }
                    return item[this._options.field] == this._options.value;
                };
                break;
            case '!=':
                filterFunc = (item) => {
                    if (['date', 'datetime', 'time'].includes(this._options.dType)) {
                        const target = moment.default.utc(item[this._options.field]);
                        const value = moment.default.utc(this._options.value);
                        return !target.isSame(value);
                    }
                    return item[this._options.field] !== this._options.value;
                };
                break;
            case 'empty':
                filterFunc = (item) => {
                    return item[this._options.field] === null;
                };
                break;
            case 'notempty':
                filterFunc = (item) => {
                    return item[this._options.field] !== null;
                };
                break;
            case 'in':
                filterFunc = (item) => {
                    const values = this._options.value;
                    return values.includes(item[this._options.field]);
                };
                break;
            case 'between':
                filterFunc = (item) => {
                    const values = this._options.value;
                    if (['date', 'datetime', 'time'].includes(this._options.dType)) {
                        const target = moment.default.utc(item[this._options.field]);
                        const lowValue = moment.default.utc(values[0]);
                        const highValue = moment.default.utc(values[1]);
                        return target.isBetween(lowValue, highValue, 'day');
                    }
                    return (item[this._options.field] > values[0] &&
                        item[this._options.field] < values[1]);
                };
                break;
            case 'startswith':
                filterFunc = (item) => {
                    return item[this._options.field].startsWith(this._options.value);
                };
                break;
            case 'endswith':
                filterFunc = (item) => {
                    return item[this._options.field].endsWith(this._options.value);
                };
                break;
            case 'stringContains':
                filterFunc = (item) => {
                    return String(item[this._options.field])
                        .toLowerCase()
                        .includes(String(this._options.value).toLowerCase());
                };
                break;
            case 'contains':
                filterFunc = (item) => {
                    return item[this._options.field].includes(this._options.value);
                };
                break;
            case '!contains':
                filterFunc = (item) => {
                    return !item[this._options.field].includes(this._options.value);
                };
                break;
            case 'isOnSameDay':
                filterFunc = (item) => {
                    const target = moment.default.utc(item[this._options.field]);
                    const value = moment.default.utc(this._options.value);
                    return target.isSame(value, 'day');
                };
                break;
            default:
                throw 'unreachable';
        }
        return {
            schema: input.schema,
            data: (0, algorithm_1.toArray)((0, algorithm_1.filter)(input.data, filterFunc)),
        };
    }
}
exports.FilterExecutor = FilterExecutor;
/**
 * A transformation that sorts the provided data by the provided field.
 *
 * Note: This is a WIP
 */
class SortExecutor extends TransformExecutor {
    /**
     * Creates a new sort transformation
     *
     * @param options - The options for initializing the transformation.
     */
    constructor(options) {
        super();
        this._options = options;
    }
    /**
     * Apply a transformation to the provided data.
     *
     * Note: Currently ignores the `desc` parameter.
     *
     * @param input - The data to be operated on.
     */
    apply(input) {
        let sortFunc;
        const field = this._options.field;
        const columnDataType = this._options.dType;
        // Adding string checks within the sort function so we do
        // not have to mutate in-place the original types of the
        // values of the column into strings. This allows the
        // displayed values to maintain their original types but
        // be sorted as if they were all strings.
        const stringifyIfNeeded = (value) => {
            if (typeof value != 'string') {
                return String(value);
            }
            return value;
        };
        if (columnDataType == 'string') {
            if (this._options.desc) {
                sortFunc = (a, b) => {
                    return stringifyIfNeeded(a[field]) < stringifyIfNeeded(b[field])
                        ? 1
                        : -1;
                };
            }
            else {
                sortFunc = (a, b) => {
                    return stringifyIfNeeded(a[field]) > stringifyIfNeeded(b[field])
                        ? 1
                        : -1;
                };
            }
        }
        else {
            if (this._options.desc) {
                sortFunc = (a, b) => {
                    return a[field] < b[field] ? 1 : -1;
                };
            }
            else {
                sortFunc = (a, b) => {
                    return a[field] > b[field] ? 1 : -1;
                };
            }
        }
        const data = input.data.slice(0);
        const sortables = [], notSortables = [];
        data.forEach((value) => {
            const cellValue = value[field];
            const notSortable = cellValue === null ||
                (typeof cellValue === 'number' && Number.isNaN(cellValue)) ||
                (cellValue instanceof Date && Number.isNaN(cellValue.getTime()));
            if (notSortable) {
                notSortables.push(value);
            }
            else {
                sortables.push(value);
            }
        });
        return {
            schema: input.schema,
            data: sortables.sort(sortFunc).concat(notSortables),
        };
    }
}
exports.SortExecutor = SortExecutor;
//# sourceMappingURL=transformExecutors.js.map