"use strict";
// Copyright (c) Bloomberg
// Distributed under the terms of the Modified BSD License.
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.VegaExprView = exports.VegaExprModel = exports.TextRendererView = exports.TextRendererModel = exports.HyperlinkRendererView = exports.HyperlinkRendererModel = exports.BarRendererView = exports.BarRendererModel = exports.DataGridView = exports.DataGridModel = void 0;
const _ = __importStar(require("underscore"));
const datagrid_1 = require("@lumino/datagrid");
const coreutils_1 = require("@lumino/coreutils");
const messaging_1 = require("@lumino/messaging");
const widgets_1 = require("@lumino/widgets");
const base_1 = require("@jupyter-widgets/base");
const viewbasedjsonmodel_1 = require("./core/viewbasedjsonmodel");
const version_1 = require("./version");
const feathergrid_1 = require("./feathergrid");
const utils_1 = require("./utils");
// Import CSS
require("../style/jupyter-widget.css");
class DataGridModel extends base_1.DOMWidgetModel {
    constructor() {
        super(...arguments);
        this.synchingWithKernel = false;
    }
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_module: DataGridModel.model_module, _model_module_version: DataGridModel.model_module_version, _view_name: DataGridModel.view_name, _view_module: DataGridModel.view_module, _view_module_version: DataGridModel.view_module_version, _visible_rows: [], _transforms: [], baseRowSize: 20, baseColumnSize: 64, baseRowHeaderSize: 64, baseColumnHeaderSize: 20, headerVisibility: 'all', _data: {}, renderers: {}, corner_renderer: null, default_renderer: null, header_renderer: null, selection_mode: 'none', selections: [], grid_style: {}, editable: false, column_widths: {} });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.updateDataSync = this.updateDataSync.bind(this);
        this.syncTransformState = this.syncTransformState.bind(this);
        // @ts-ignore
        this.on('change:_data', this.updateData.bind(this));
        // @ts-ignore
        this.on('change:_transforms', this.updateTransforms.bind(this));
        // @ts-ignore
        this.on('change:selection_mode', this.updateSelectionModel, this);
        // @ts-ignore
        this.on('change:selections', this.updateSelections, this);
        this.updateData();
        this.updateTransforms();
        this.updateSelectionModel();
        // @ts-ignore
        this.on('msg:custom', (content) => {
            if (content.event_type === 'cell-changed') {
                this.data_model.setModelData('body', content.row, content.column_index, content.value);
            }
            if (content.event_type === 'row-changed') {
                this.data_model.setRowData(content.row, content.value);
            }
        });
    }
    updateDataSync(sender, msg) {
        switch (msg.type) {
            case 'row-indices-updated':
                this.set('_visible_rows', msg.indices);
                this.save_changes();
                break;
            case 'cell-updated':
                this.set('_data', this.data_model.dataset);
                this.save_changes();
                break;
            case 'cell-edit-event':
                // Update data in widget model
                // @ts-ignore
                const newData = this.get('_data');
                newData.data[msg.row][msg.columnIndex] = msg.value;
                this.set('_data', newData);
                this.send({
                    event_type: 'cell-changed',
                    region: msg.region,
                    row: msg.row,
                    column_index: msg.columnIndex,
                    value: msg.value,
                }, Object.assign({}, this._view_callbacks));
                break;
            default:
                throw 'unreachable';
        }
    }
    syncTransformState(sender, value) {
        this.set('_transforms', value.transforms);
        this.save_changes();
    }
    updateData() {
        const data = this.data;
        const schema = Private.createSchema(data);
        if (this.data_model) {
            this.data_model.updateDataset({ data: data.data, schema: schema });
            this.data_model.transformStateChanged.disconnect(this.syncTransformState);
            this.data_model.dataSync.disconnect(this.updateDataSync);
        }
        this.data_model = new viewbasedjsonmodel_1.ViewBasedJSONModel({
            data: data.data,
            schema: schema,
        });
        this.data_model.transformStateChanged.connect(this.syncTransformState);
        this.data_model.dataSync.connect(this.updateDataSync);
        this.updateTransforms();
        // @ts-ignore
        this.trigger('data-model-changed');
        this.updateSelectionModel();
    }
    updateTransforms() {
        if (this.selectionModel) {
            this.selectionModel.clear();
        }
        // @ts-ignore
        this.data_model.replaceTransforms(this.get('_transforms'));
    }
    updateSelectionModel() {
        if (this.selectionModel) {
            this.selectionModel.clear();
        }
        // @ts-ignore
        const selectionMode = this.get('selection_mode');
        if (selectionMode === 'none') {
            this.selectionModel = null;
            return;
        }
        this.selectionModel = new datagrid_1.BasicSelectionModel({
            dataModel: this.data_model,
        });
        this.selectionModel.selectionMode = selectionMode;
        // @ts-ignore
        this.trigger('selection-model-changed');
        this.selectionModel.changed.connect((sender, args) => {
            if (this.synchingWithKernel) {
                return;
            }
            this.synchingWithKernel = true;
            const selections = [];
            let selectionIter = sender.selections();
            // @ts-ignore
            if (typeof selectionIter.iter === 'function') {
                // Lumino 1 (JupyterLab 3)
                let selection = null;
                // @ts-ignore
                selectionIter = selectionIter.iter();
                while ((selection = selectionIter.next())) {
                    selections.push({
                        // @ts-ignore
                        r1: Math.min(selection.r1, selection.r2),
                        // @ts-ignore
                        r2: Math.max(selection.r1, selection.r2),
                        // @ts-ignore
                        c1: Math.min(selection.c1, selection.c2),
                        // @ts-ignore
                        c2: Math.max(selection.c1, selection.c2),
                    });
                }
            }
            else {
                // Lumino 2 (JupyterLab 4)
                let selectionNode = null;
                while (selectionNode = selectionIter.next()) {
                    if (selectionNode.done) {
                        break;
                    }
                    const selection = selectionNode.value;
                    selections.push({
                        r1: Math.min(selection.r1, selection.r2),
                        r2: Math.max(selection.r1, selection.r2),
                        c1: Math.min(selection.c1, selection.c2),
                        c2: Math.max(selection.c1, selection.c2),
                    });
                }
            }
            this.set('selections', selections);
            this.save_changes();
            this.synchingWithKernel = false;
        }, this);
    }
    updateSelections() {
        if (!this.selectionModel || this.synchingWithKernel) {
            return;
        }
        this.synchingWithKernel = true;
        // @ts-ignore
        const selections = this.get('selections');
        this.selectionModel.clear();
        for (const selection of selections) {
            this.selectionModel.select({
                r1: selection.r1,
                c1: selection.c1,
                r2: selection.r2,
                c2: selection.c2,
                cursorRow: selection.r1,
                cursorColumn: selection.c1,
                clear: 'none',
            });
        }
        this.synchingWithKernel = false;
    }
    get data() {
        // @ts-ignore
        return this.get('_data');
    }
}
exports.DataGridModel = DataGridModel;
DataGridModel.serializers = Object.assign(Object.assign({}, base_1.DOMWidgetModel.serializers), { transforms: { deserialize: base_1.unpack_models }, renderers: { deserialize: base_1.unpack_models }, corner_renderer: { deserialize: base_1.unpack_models }, default_renderer: { deserialize: base_1.unpack_models }, header_renderer: { deserialize: base_1.unpack_models }, _data: { deserialize: unpack_data }, grid_style: { deserialize: unpack_style } });
DataGridModel.model_name = 'DataGridModel';
DataGridModel.model_module = version_1.MODULE_NAME;
DataGridModel.model_module_version = version_1.MODULE_VERSION;
DataGridModel.view_name = 'DataGridView';
DataGridModel.view_module = version_1.MODULE_NAME;
DataGridModel.view_module_version = version_1.MODULE_VERSION;
/**
 * Helper function to conver snake_case strings to camelCase.
 * Assumes all strings are valid snake_case (all lowercase).
 * @param string snake_case string
 * @returns camelCase string
 */
function camelCase(string) {
    string = string.toLowerCase();
    const charArray = [];
    for (let i = 0; i < string.length; i++) {
        const curChar = string.charAt(i);
        if (curChar === '_') {
            i++;
            charArray.push(string.charAt(i).toUpperCase());
            continue;
        }
        charArray.push(curChar);
    }
    return charArray.join('');
}
/**
 * Custom deserialization function for grid styles.
 */
function unpack_style(value, manager) {
    if (value instanceof Object && typeof value !== 'string') {
        const unpacked = {};
        Object.keys(value).forEach((key) => {
            unpacked[camelCase(key)] = unpack_style(value[key], manager);
        });
        return (0, base_1.resolvePromisesDict)(unpacked);
    }
    else if (typeof value === 'string' && value.slice(0, 10) === 'IPY_MODEL_') {
        return Promise.resolve(manager.get_model(value.slice(10, value.length))).then((model) => {
            // returning the color formatting function from VegaExprModel.
            return model._function;
        });
    }
    else {
        return Promise.resolve(value);
    }
}
// modified from ipywidgets original
function unpack_data(value, manager) {
    if (Array.isArray(value)) {
        const unpacked = [];
        value.forEach((sub_value, key) => {
            unpacked.push(unpack_data(sub_value, manager));
        });
        return Promise.all(unpacked);
    }
    else if (value instanceof Object && typeof value !== 'string') {
        const unpacked = {};
        Object.keys(value).forEach((key) => {
            unpacked[key] = unpack_data(value[key], manager);
        });
        return (0, base_1.resolvePromisesDict)(unpacked);
    }
    else if (value === '$NaN$') {
        return Promise.resolve(Number.NaN);
    }
    else if (value === '$Infinity$') {
        return Promise.resolve(Number.POSITIVE_INFINITY);
    }
    else if (value === '$NegInfinity$') {
        return Promise.resolve(Number.NEGATIVE_INFINITY);
    }
    else if (value === '$NaT$') {
        return Promise.resolve(new Date('INVALID'));
    }
    else {
        return Promise.resolve(value);
    }
}
class DataGridView extends base_1.DOMWidgetView {
    constructor() {
        super(...arguments);
        this.manageResizeEvent = () => {
            messaging_1.MessageLoop.postMessage(this.luminoWidget, widgets_1.Widget.ResizeMessage.UnknownSize);
        };
    }
    _createElement(tagName) {
        const panelWidget = Private.getWidgetPanel();
        this.luminoWidget = new panelWidget({ view: this });
        this._initializeTheme();
        return this.luminoWidget.node;
    }
    // @ts-ignore Added for ipywidgets 7.x compatibility
    get pWidget() {
        return this.luminoWidget;
    }
    _setElement(el) {
        if (this.el || el !== this.luminoWidget.node) {
            throw new Error('Cannot reset the DOM element.');
        }
        this.el = this.luminoWidget.node;
    }
    // ipywidgets 7 compatibility
    _processLuminoMessage(msg, _super) {
        _super.call(this, msg);
        switch (msg.type) {
            case 'after-show':
                if (this.luminoWidget.isVisible) {
                    this.manageResizeEvent();
                }
                break;
        }
    }
    processLuminoMessage(msg) {
        // @ts-ignore needed for ipywidgets 8.x compatibility
        this._processLuminoMessage(msg, super.processLuminoMessage);
    }
    processPhosphorMessage(msg) {
        // @ts-ignore needed for ipywidgets 7.x compatibility
        this._processLuminoMessage(msg, super.processPhosphorMessage);
    }
    render() {
        this.el.classList.add('datagrid-container');
        window.addEventListener('resize', this.manageResizeEvent);
        // @ts-ignore
        this.once('remove', () => {
            window.removeEventListener('resize', this.manageResizeEvent);
        });
        this.grid = new feathergrid_1.FeatherGrid({
            defaultSizes: {
                // @ts-ignore
                rowHeight: this.model.get('base_row_size'),
                // @ts-ignore
                columnWidth: this.model.get('base_column_size'),
                // @ts-ignore
                rowHeaderWidth: this.model.get('base_row_header_size'),
                // @ts-ignore
                columnHeaderHeight: this.model.get('base_column_header_size'),
            },
            // @ts-ignore
            headerVisibility: this.model.get('header_visibility'),
            // @ts-ignore
            style: this.model.get('grid_style'),
        });
        // @ts-ignore
        this.grid.columnWidths = this.model.get('column_widths');
        // @ts-ignore
        this.grid.editable = this.model.get('editable');
        // this.default_renderer must be created after setting grid.isLightTheme
        // for proper color variable initialization
        this.grid.isLightTheme = this._isLightTheme;
        this.grid.dataModel = this.model.data_model;
        this.grid.selectionModel = this.model.selectionModel;
        this.grid.backboneModel = this.model;
        this.grid.cellClicked.connect((sender, event) => {
            if (this.model.comm) {
                this.send({
                    event_type: 'cell-click',
                    region: event.region,
                    column: event.column,
                    column_index: event.columnIndex,
                    row: event.row,
                    primary_key_row: event.primaryKeyRow,
                    cell_value: event.cellValue,
                });
            }
        });
        this.grid.columnsResized.connect((sender, args) => {
            this.model.set('column_widths', coreutils_1.JSONExt.deepCopy(this.grid.columnWidths));
            this.model.save_changes();
        });
        // Attaching the view's iopub callbacks functions to
        // the data model so we can use those as an
        // argument to model.send() function in the model class.
        this.model._view_callbacks = this.callbacks();
        // @ts-ignore
        this.model.on('data-model-changed', () => {
            this.grid.dataModel = this.model.data_model;
            this.handleColumnAutoFit();
        });
        // @ts-ignore
        this.model.on('change:base_row_size', () => {
            // @ts-ignore
            this.grid.baseRowSize = this.model.get('base_row_size');
        });
        // @ts-ignore
        this.model.on('change:base_column_size', () => {
            // @ts-ignore
            this.grid.baseColumnSize = this.model.get('base_column_size');
        });
        // @ts-ignore
        this.model.on('change:column_widths', () => {
            this.grid.columnWidths = this.model.get('column_widths');
        });
        // @ts-ignore
        this.model.on('change:base_row_header_size', () => {
            // @ts-ignore
            this.grid.baseRowHeaderSize = this.model.get('base_row_header_size');
        });
        // @ts-ignore
        this.model.on('change:base_column_header_size', () => {
            // @ts-ignore
            this.grid.baseColumnHeaderSize = this.model.get('base_column_header_size');
        });
        // @ts-ignore
        this.model.on('change:header_visibility', () => {
            this.grid.headerVisibility = this.model.get('header_visibility');
        });
        // @ts-ignore
        this.model.on_some_change([
            'corner_renderer',
            'header_renderer',
            'default_renderer',
            'renderers',
            'grid_style',
        ], () => {
            this.updateRenderers()
                .then(this.updateGridStyle.bind(this))
                .then(this.updateGridRenderers.bind(this));
        }, this);
        // @ts-ignore
        this.model.on('selection-model-changed', () => {
            this.grid.selectionModel = this.model.selectionModel;
        });
        // @ts-ignore
        // @ts-ignore
        this.model.on('change:editable', () => {
            // @ts-ignore
            this.grid.editable = this.model.get('editable');
        });
        this.model.on_some_change(['auto_fit_columns', 'auto_fit_params'], () => {
            this.handleColumnAutoFit();
        }, this);
        // @ts-ignore
        if (this.model.get('auto_fit_columns')) {
            this.handleColumnAutoFit();
        }
        return this.updateRenderers().then(() => {
            this.updateGridStyle();
            this.updateGridRenderers();
            this.luminoWidget.addWidget(this.grid);
        });
    }
    handleColumnAutoFit() {
        // Check whether we need to auto-fit or revert to base size.
        // @ts-ignore
        const shouldAutoFit = this.model.get('auto_fit_columns');
        if (!shouldAutoFit) {
            // @ts-ignore
            this.grid.baseColumnSize = this.model.get('base_column_size');
            // Terminate call here if not auto-fitting.
            return;
        }
        // Retrieve user-defined auto-fit params
        // @ts-ignore
        let { area, padding, numCols } = this.model.get('auto_fit_params');
        // Data validation on params
        area = area !== null && area !== void 0 ? area : 'all';
        padding = padding !== null && padding !== void 0 ? padding : 30;
        numCols = numCols !== null && numCols !== void 0 ? numCols : undefined;
        // Call resize function
        this.grid.grid.fitColumnNames(area, padding, numCols);
    }
    updateRenderers() {
        // Unlisten to previous renderers
        if (this.default_renderer) {
            // @ts-ignore
            this.stopListening(this.default_renderer, 'renderer-changed');
        }
        if (this.header_renderer) {
            // @ts-ignore
            this.stopListening(this.header_renderer, 'renderer-changed');
        }
        if (this.corner_renderer) {
            // @ts-ignore
            this.stopListening(this.corner_renderer, 'renderer-changed');
        }
        for (const key in this.renderers) {
            // @ts-ignore
            this.stopListening(this.renderers[key], 'renderer-changed');
        }
        // And create views for new renderers
        const promises = [];
        // @ts-ignore
        const default_renderer = this.model.get('default_renderer');
        promises.push(this.create_child_view(default_renderer).then((defaultRendererView) => {
            this.default_renderer = defaultRendererView;
            // @ts-ignore
            this.listenTo(this.default_renderer, 'renderer-changed', this.updateGridRenderers.bind(this));
        }));
        // @ts-ignore
        const corner_renderer = this.model.get('corner_renderer');
        if (corner_renderer) {
            promises.push(this.create_child_view(corner_renderer).then((cornerRendererView) => {
                this.corner_renderer = cornerRendererView;
                // @ts-ignore
                this.listenTo(this.corner_renderer, 'renderer-changed', this.updateGridRenderers.bind(this));
            }));
        }
        // @ts-ignore
        const header_renderer = this.model.get('header_renderer');
        if (header_renderer) {
            promises.push(this.create_child_view(header_renderer).then((headerRendererView) => {
                this.header_renderer = headerRendererView;
                // @ts-ignore
                this.listenTo(this.header_renderer, 'renderer-changed', this.updateGridRenderers.bind(this));
            }));
        }
        const renderer_promises = {};
        _.each(
        // @ts-ignore
        this.model.get('renderers'), (model, key) => {
            renderer_promises[key] = this.create_child_view(model);
        });
        promises.push((0, base_1.resolvePromisesDict)(renderer_promises).then((rendererViews) => {
            this.renderers = rendererViews;
            for (const key in rendererViews) {
                // @ts-ignore
                this.listenTo(rendererViews[key], 'renderer-changed', this.updateGridRenderers.bind(this));
            }
        }));
        return Promise.all(promises);
    }
    updateGridStyle() {
        this.grid.updateGridStyle();
    }
    set isLightTheme(value) {
        this._isLightTheme = value;
        if (!this.grid) {
            return;
        }
        this.grid.isLightTheme = value;
    }
    get isLightTheme() {
        return this._isLightTheme;
    }
    updateGridRenderers() {
        let defaultRenderer = this.default_renderer.renderer;
        if (this.grid.grid.style.backgroundColor !== utils_1.Theme.getBackgroundColor() ||
            this.grid.grid.style.rowBackgroundColor ||
            this.grid.grid.style.columnBackgroundColor) {
            // Making sure the default renderer doesn't paint over the global
            // grid background color, if the latter is set.
            defaultRenderer = new datagrid_1.TextRenderer(Object.assign(Object.assign({}, this.default_renderer.renderer), { backgroundColor: undefined }));
        }
        // @ts-ignore
        const hasHeaderRenderer = this.model.get('header_renderer') !== null;
        let columnHeaderRenderer = null;
        if (this.header_renderer && hasHeaderRenderer) {
            columnHeaderRenderer = this.header_renderer.renderer;
        }
        else {
            columnHeaderRenderer = new datagrid_1.TextRenderer({
                font: '12px sans-serif',
                textColor: utils_1.Theme.getFontColor(),
                backgroundColor: utils_1.Theme.getBackgroundColor(2),
            });
        }
        let cornerHeaderRenderer = null;
        if (this.corner_renderer) {
            cornerHeaderRenderer = this.corner_renderer.renderer;
        }
        const renderers = {};
        Object.entries(this.renderers).forEach(([name, rendererView]) => {
            renderers[name] = rendererView.renderer;
        });
        this.grid.defaultRenderer = defaultRenderer;
        this.grid.columnHeaderRenderer = columnHeaderRenderer;
        if (cornerHeaderRenderer) {
            this.grid.cornerHeaderRenderer = cornerHeaderRenderer;
        }
        this.grid.renderers = renderers;
    }
    _initializeTheme() {
        // initialize theme unless set earlier
        if (this._isLightTheme !== undefined) {
            return;
        }
        // initialize theme based on application settings
        this._isLightTheme = !((document.body.classList.contains('theme-dark') /* jupyter notebook or voila */ ||
            document.body.dataset.jpThemeLight === 'false') /* jupyter lab */);
    }
}
exports.DataGridView = DataGridView;
var cellrenderer_1 = require("./cellrenderer");
Object.defineProperty(exports, "BarRendererModel", { enumerable: true, get: function () { return cellrenderer_1.BarRendererModel; } });
Object.defineProperty(exports, "BarRendererView", { enumerable: true, get: function () { return cellrenderer_1.BarRendererView; } });
Object.defineProperty(exports, "HyperlinkRendererModel", { enumerable: true, get: function () { return cellrenderer_1.HyperlinkRendererModel; } });
Object.defineProperty(exports, "HyperlinkRendererView", { enumerable: true, get: function () { return cellrenderer_1.HyperlinkRendererView; } });
Object.defineProperty(exports, "TextRendererModel", { enumerable: true, get: function () { return cellrenderer_1.TextRendererModel; } });
Object.defineProperty(exports, "TextRendererView", { enumerable: true, get: function () { return cellrenderer_1.TextRendererView; } });
var vegaexpr_1 = require("./vegaexpr");
Object.defineProperty(exports, "VegaExprModel", { enumerable: true, get: function () { return vegaexpr_1.VegaExprModel; } });
Object.defineProperty(exports, "VegaExprView", { enumerable: true, get: function () { return vegaexpr_1.VegaExprView; } });
/**
 * The namespace for the module implementation details.
 */
var Private;
(function (Private) {
    function getWidgetPanel() {
        // @ts-ignore needed for ipywidget 7.x compatibility
        return base_1.JupyterLuminoPanelWidget !== null && base_1.JupyterLuminoPanelWidget !== void 0 ? base_1.JupyterLuminoPanelWidget : base_1.JupyterPhosphorPanelWidget;
    }
    Private.getWidgetPanel = getWidgetPanel;
    /**
     * Creates a valid JSON Table Schema from the schema provided by pandas.
     *
     * @param data - The data that has been synced from the kernel.
     */
    function createSchema(data) {
        // Construct a new array of schema fields based on the keys in data.fields
        // Note: this accounts for how tuples/lists may be serialized into strings
        // in the case of multiIndex columns.
        const fields = [];
        data.fields.forEach((val, i) => {
            const rows = Array.isArray(data.schema.fields[i].name)
                ? data.schema.fields[i].name
                : [data.schema.fields[i].name];
            const field = {
                name: Object.keys(val)[0],
                type: data.schema.fields[i].type,
                rows: rows,
            };
            fields.push(field);
        });
        // Updating the primary key to account for a multiIndex primary key.
        const primaryKey = data.schema.primaryKey.map((key) => {
            for (let i = 0; i < data.schema.fields.length; i++) {
                const curFieldKey = Array.isArray(key)
                    ? data.schema.fields[i].name[0]
                    : data.schema.fields[i].name;
                const newKey = Array.isArray(key) ? key[0] : key;
                if (curFieldKey == newKey) {
                    return Object.keys(data.fields[i])[0];
                }
            }
            return 'unreachable';
        });
        return {
            primaryKey: primaryKey,
            primaryKeyUuid: data.schema.primaryKeyUuid,
            fields: fields,
        };
    }
    Private.createSchema = createSchema;
})(Private || (Private = {}));
//# sourceMappingURL=datagrid.js.map